#include "../window.hpp"

#include "libobsensor/hpp/Pipeline.hpp"
#include "libobsensor/hpp/Error.hpp"

int main(int argc, char **argv) try {
    //创建一个Pipeline，Pipeline是整个高级API的入口，通过Pipeline可以很容易的打开和关闭
    //多种类型的流并获取一组帧数据
    ob::Pipeline pipe;

    //获取彩色相机的所有流配置，包括流的分辨率，帧率，以及帧的格式
    auto profiles = pipe.getStreamProfileList(OB_SENSOR_COLOR);
    //通过遍历彩色流的配置来获取要使用的帧格式，这里将使用MJPG格式来打开流
    std::shared_ptr<ob::VideoStreamProfile> colorProfile = nullptr;
    for(int i = 0; i < profiles->count(); i++) {
        auto profile = profiles->getProfile(i)->as<ob::VideoStreamProfile>();
        if(profile->format() == OB_FORMAT_MJPG) {
            colorProfile = profile;
            break;
        }
    }
    //也可以通过接口设置感兴趣项，返回对应Profile列表的首个Profile
    // auto colorProfile = profiles->getVideoStreamProfile(640,480,OB_FORMAT_MJPG);

    //如果没找到合适的流配置，打开列表中的首个配置
    if(!colorProfile) {
        colorProfile = profiles->getProfile(0)->as<ob::VideoStreamProfile>();
    }

    //通过创建Config来配置Pipeline要启用或者禁用哪些流，这里将启用彩色流
    std::shared_ptr<ob::Config> config = std::make_shared<ob::Config>();
    config->enableStream(colorProfile);

    //启动在Config中配置的流，如果不传参数，将启动默认配置启动流
    pipe.start(config);

    //创建一个用于渲染的窗口，并设置窗口的分辨率
    Window app("ColorViewer", colorProfile->width(), colorProfile->height());

    //获取镜像属性是否有可写的权限
    if(pipe.getDevice()->isPropertySupported(OB_PROP_COLOR_MIRROR_BOOL, OB_PERMISSION_WRITE)) {
        //设置镜像
        pipe.getDevice()->setBoolProperty(OB_PROP_COLOR_MIRROR_BOOL, true);
    }

    while(app) {
        //以阻塞的方式等待一帧数据，该帧是一个复合帧，里面包含配置里启用的所有流的帧数据，
        //并设置帧的等待超时时间为100ms
        auto frameSet = pipe.waitForFrames(100);
        if(frameSet == nullptr) {
            continue;
        }

        //在窗口中渲染一组帧数据，这里只渲染彩色帧，但是也要以数组的形式传入，RENDER_SINGLE表示
        //只渲染数组中的第一个帧
        app.render({ frameSet->colorFrame() }, RENDER_SINGLE);
    }

    //停止Pipeline，将不再产生帧数据
    pipe.stop();

    return 0;
}
catch(ob::Error &e) {
    std::cerr << "function:" << e.getName() << "\nargs:" << e.getArgs() << "\nmessage:" << e.getMessage() << "\ntype:" << e.getExceptionType() << std::endl;
    exit(EXIT_FAILURE);
}
